(function (ng) {
    /**
     * @ngdoc service
     * @module apeman-ui-contrib-angular-locating
     * @name apLocationSearch
     * @description Location search wrapper.
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-locating')
        .service('apLocationSearch', function ($location, apObjectNest) {
            var s = this;
            /**
             * @ngdoc method
             * @name apLocationSearch#consume
             * @description Get location search with key and delete the key from search.
             * @param {string} key - Search key.
             * @returns {string} - Search value with the key.
             */
            s.consume = function (key) {
                var search = $location.search(),
                    val = search[key];
                s.delete(key);
                return val;
            };

            /**
             * @ngdoc method
             * @name apLocationSearch#delete
             * @description Delete search with pattern.
             * @param {string} key - Key to delete.
             */
            s.delete = function (key) {
                var search = s.get();
                delete search[key];
                $location.search(search);
            };


            /**
             * @ngdoc method
             * @name apLocationSearch#deleteByPattern
             * @description Delete search with pattern.
             * @param {string|RegExp} pattern - Pattern to compare.
             */
            s.deleteByPattern = function (pattern) {
                var search = apObjectNest.flatten(s.get());
                Object.keys(search).forEach(function (key) {
                    var shouldDelete = key.match(pattern);
                    if (shouldDelete) {
                        delete search[key];
                    }
                });
                $location.search(search);
            };

            /**
             * @ngdoc method
             * @name apLocationSearch#get
             * @description Get search type.
             * @param {string} key - Key to search.
             * @returns {string|object} - Search value
             */
            s.get = function (key) {
                var search = $location.search();
                if (arguments.length == 0) {
                    return apObjectNest.expand(search);
                }
                if (search.hasOwnProperty(key)) {
                    return search[key];
                }
                var values = apObjectNest.expand(search),
                    keys = key.split(/\./g);
                for (var i = 0; i < keys.length; i++) {
                    var shortKey = keys[i];
                    if (!values.hasOwnProperty(shortKey)) {
                        return undefined;
                    }
                    values = values[shortKey];
                }
                return values;
            };

            /**
             * @ngdoc method
             * @name apLocationSearch#getByPattern
             * @description Get search by pattern
             * @param pattern
             * @returns {object} values.
             */
            s.getByPattern = function (pattern) {
                var search = apObjectNest.expand($location.search() || {}),
                    result = {};
                Object.keys(search).forEach(function (key) {
                    var shouldGet = key.match(pattern);
                    if (shouldGet) {
                        result[key] = search[key];
                    }
                });
                return result;
            };


            /**
             * @ngdoc method
             * @name apLocationSearch#set
             * @description Change search value.
             * @param {string} key - Key to set.
             * @param {string} val - Value to set.
             */
            s.set = function (key, val) {
                var values;
                if (arguments.length == 2) {
                    values = {};
                    values[key] = val;
                    s.set(values);
                    return;
                }
                values = apObjectNest.flatten(arguments[0]);
                var search = $location.search();
                Object.keys(values).forEach(function (key) {
                    search[key] = values[key];
                });
                $location.search(search);
            };
        });

})(angular);