(function (ng) {
    /**
     * @ngdoc module
     * @module apeman-ui-contrib-angular-objecting
     * @name apeman-ui-contrib-angular-objecting
     * @description
     * apeman ui module for angular to work with object.
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-objecting', [
        ]);
})(angular);(function (ng) {

    /**
     * @ngdoc service
     * @module apeman-ui-contrib-angular-objecting
     * @description Utility for coping objects.
     * @name apObjectCopy
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-objecting')
        .service('apObjectCopy', function () {
            var s = this;

            /**
             * @ngdoc method
             * @name apObjectCopy#copy
             * @description Copy an object.
             * @param {object} src - Object copy from
             * @param {object} dest - Object copy to.
             * @returns {dest} - Destination object.
             */
            s.copy = function (src, dest) {
                var keys = Object.keys(src);
                for (var i = 0, len = keys.length; i < len; i++) {
                    var key = keys[i];
                    dest[key] = src[key];
                }
                return dest;
            }
        });

})(angular);(function (ng) {

    /**
     * @ngdoc service
     * @module apeman-ui-contrib-angular-objecting
     * @description Utility for handle nested objects.
     * @name apObjectNest
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-objecting')
        .service('apObjectNest', function (apObjectCopy) {

            var copy = apObjectCopy.copy.bind(apObjectCopy);

            var s = this;
            /**
             * @ngdoc method
             * @name apObjectNest#expand
             * @description Expand nested object.
             * @param {object} obj - Obj to flatten
             * @returns {object} Object expanded.
             * @example
             *  var obj = expandNested({
             *      'foo.bar': 'baz'
             * });
             * console.log(obj); // {foo: {bar: 'baz'}}
             */
            s.expand = function (obj) {
                var result = {};
                Object.keys(obj).forEach(function (key) {
                    var val = obj[key];
                    if (/\./g.test(key)) {
                        var subKeys = key.split(/\./g),
                            subObj = {},
                            thisKey = subKeys.shift();
                        subObj[subKeys.join('.')] = val;
                        var subExpandedObj = s.expand(subObj);
                        result[thisKey] = copy(subExpandedObj, result[thisKey] || {});
                    } else {
                        result[key] = val;
                    }
                });
                return result;
            };

            /**
             * @ngdoc method
             * @name apObjectNest#flatten
             * @description Flatten nested object.
             * @param {object} obj - Object to flatten.
             * @returns {object} - Flattened object.
             * @example
             * var obj = flatten({
             *      foo: {bar: 'baz'}
            * });
             * console.log(obj); // {'foo.bar': 'baz'}
             */
            s.flatten = function (obj) {
                var result = {};
                Object.keys(obj || {}).forEach(function (key) {
                    var val = obj[key];
                    var isObj = typeof(val) === 'object';
                    if (isObj) {
                        var subValues = s.flatten(val);
                        Object.keys(subValues).forEach(function (subKey) {
                            var fullKey = [key, subKey].join('.');
                            result[fullKey] = subValues[subKey];
                        });
                        return;
                    }
                    result[key] = val;
                });
                return result;
            };

        });

})(angular);